#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : single.py
# Contents: detect synchronous spiking activity and reduce results
# Author  : Christian Borgelt
# History : 2013.01.29 file created from detect.py
#           2013.02.09 made compatible with Python 3 (print, range)
#           2014.05.27 modules placed in sibling directories
#-----------------------------------------------------------------------
from sys     import argv, stderr, path
from os      import makedirs
from os.path import join
from random  import seed
NEURODIR = join('..', 'neuro')
if NEURODIR not in path: path.insert(0, NEURODIR)
from psts    import genpsts, psts2tras
from fim     import fim, accretion

#-----------------------------------------------------------------------

oo = float('inf')               # positive infinity as a constant

#-----------------------------------------------------------------------

def single (ncnt=100, period=3, rate=20,  # neuron   parameters
            size=5, coin=5, width=0.003): # assembly parameters
    base = 'n%d_p%g_r%g_w%g_c' % (ncnt,period,rate,width*1000)
    psp  = set()                # initialize a pattern spectrum
    with open(join('data','pc_'+base+'.txt'), 'r') as inp:
        for line in inp:        # collect the pattern signatures
            z,c,n = line.split()
            psp.add((int(z),int(c)))
    border = list(range(ncnt+1)); m = 1
    for z in reversed(border):  # determine the detection border
        m = max([c for a,c in psp if a == z]+[m])
        border[z] = m+1         # (support thresholds per size)
    border[0:2] = [oo,oo]       # entirely rule out sizes 0 and 1
    asm  = set(range(1,size+1)) # create the assembly to inject
    psts = genpsts(ncnt, period, rate, asms=[[asm,-coin]])
    tras = psts2tras(psts, width, period)
    pats = fim(tras, target='c', supp=-2, zmin=2)
    fltd = []                   # generate and mine spike trains
    for p,c in pats:            # and traverse the found patterns
        z = len(p); c = c[0]    # collect patterns that are not excluded
        if c >= border[z]: fltd.append((tuple(sorted(p)),c))
    pats = [c for a,b,c in sorted([(len(p[0]),p[1],p) for p in fltd])]
    with open(join('data','fim.txt'), 'w') as out:
        for p,c in reversed(pats): # write filtered and sorted patterns
            out.write(str(p) +' : '+str(c)+'\n')
    for maxext in [2,-1]:       # run (default) accretion on same data
        pacc = accretion(tras, maxext=maxext, invbxs=1)
        pacc = set([(tuple(sorted(p)),r[0]) for p,r in pacc])
        pacc = sorted([(len(p[0]),p[1],p) for p in pacc])
        pacc = [c for a,b,c in pacc]
        fltd = []               # make accretion results unique
        for p,c in pacc:        # traverse the accretion results
            q = set(p)          # and categorize them
            if   q == asm:          x = 0
            elif q >  asm:          x = 1
            elif q <  asm:          x = 2
            elif len(q & asm) >= 2: x = 3
            else:                   x = 4
            fltd.append((x,p,c))# append categorized results
        pacc = sorted(fltd)     # and sort them by category
        name = 'acx.txt' if maxext < 0 else 'acc.txt'
        cats = ['=', '>', '<', 'o', '#']
        with open(join('data', name), 'w') as out:
            for x,p,c in pacc:  # write filtered and sorted patterns
                out.write(cats[x] +' ' +str(p) +' : ' +str(c) +'\n')
    return pats                 # return the found patterns

#-----------------------------------------------------------------------

if __name__ == '__main__':
    # 145 270* 2186 3863 4327 6091 6366 7610 8916 9341 9386
    s = int(argv[1]) if len(argv) > 1 else 270
    for i in range(s,10000):    # search for a proper data set
        seed(i)                 # init. the random number generator
        pats = single(100, 3, 20, 7, 7, 0.003)
        x = 0                   # check patterns for specific properties
        for p,c in pats:        # (for proper demonstration)
            if   len(p) >= 9:            x |=  1
            elif len(p) >= 8:            x |=  2
            elif len(p) <  7 and c <  7: x |=  4
            elif len(p) == 4 and c >= 8: x |=  8
            elif len(p) == 3 and c >= 9: x |= 16
            elif len(p) == 3 and c == 8: x |= 32
        stderr.write('%6d %2d\b\b\b\b\b\b\b\b\b' % (i,x))
        if x >= 63: break
    stderr.write('\n')          # terminate the output
